/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlCast.h"
#include "HvSignalVar.h"
#include "HvSignalEnvelope.h"
#include "HvSignalLine.h"
#include "HvControlVar.h"
#include "HvControlBinop.h"
#include "HvControlIf.h"
#include "HvMath.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_M8rZRWN6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_2NQ20GYI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_LaUmiSSy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_qwkoYCvx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_qR7moa9M_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Kx0kXZpO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_0LbFAYm9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_2SetZZj3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_U1PLObQa_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_6gBflQY5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_HjukW9Wu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_LYXCUfo6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_if5JHXpY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_YqOP3jXM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_UOMlKGrO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_gsVNt7H6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_5vRENgxp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_UNFtNqlo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_mQEoD6Re_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ra9nBoGt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_tlkl91Ku_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_3SP9QV5z_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Wgx402fM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_VhufbP5z_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_BDyvzTNg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_w9b6MFHE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Zf0AbBu6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_rUQUEBiz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_05WP8nwu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_X5gXPnHO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Zrt41spu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_MR00YWwS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_iJUHHHqY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_oVr9O1zG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_iQ7EE2zh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_52J2LowK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_WtHtDq81_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_bpPIoCGg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_3FOiZZ2P_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_3DP7AbBA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_RnnnzuP1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_8310c1YH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_fZ6WLDOL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_kLEUfcoV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_jhdmURhV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_YvN3bTcP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_lbdDoIkp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_4mWAPeAQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_2PCytTXP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_fBJsEGdX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_bYrvj78i_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_aUc8hrBO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_J6b3MBTO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_dAXTkyMW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_j02eTMNI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_uHdxhTwm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_3zktFVF6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_o7MuLb4y_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_OK4DyhSe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_gmcFvotD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_blDHW94Z_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_deyQ7fbb_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_uRPWk667;
  SignalEnvelope sEnv_UNFtNqlo;
  SignalEnvelope sEnv_mQEoD6Re;
  SignalEnvelope sEnv_iQ7EE2zh;
  SignalEnvelope sEnv_52J2LowK;
  SignalLine sLine_M5HMpY6l;
  SignalLine sLine_kXye73ww;
  ControlBinop cBinop_M8rZRWN6;
  ControlBinop cBinop_2NQ20GYI;
  ControlBinop cBinop_LaUmiSSy;
  ControlBinop cBinop_qwkoYCvx;
  ControlBinop cBinop_0LbFAYm9;
  ControlBinop cBinop_U1PLObQa;
  ControlBinop cBinop_6gBflQY5;
  ControlBinop cBinop_HjukW9Wu;
  ControlVar cVar_LYXCUfo6;
  ControlVar cVar_YqOP3jXM;
  SignalVarf sVarf_j6LCDSq8;
  ControlVar cVar_UOMlKGrO;
  ControlIf cIf_gsVNt7H6;
  ControlIf cIf_5vRENgxp;
  ControlBinop cBinop_ra9nBoGt;
  ControlBinop cBinop_3SP9QV5z;
  ControlBinop cBinop_Wgx402fM;
  ControlBinop cBinop_w9b6MFHE;
  ControlBinop cBinop_rUQUEBiz;
  SignalVarf sVarf_Mg57txmH;
  ControlVar cVar_MR00YWwS;
  ControlIf cIf_iJUHHHqY;
  ControlIf cIf_oVr9O1zG;
  ControlBinop cBinop_WtHtDq81;
  ControlBinop cBinop_3FOiZZ2P;
  ControlBinop cBinop_3DP7AbBA;
  ControlBinop cBinop_fZ6WLDOL;
  ControlBinop cBinop_jhdmURhV;
  ControlVar cVar_2PCytTXP;
  ControlBinop cBinop_fBJsEGdX;
  SignalVarf sVarf_mZaqsQrM;
  SignalVarf sVarf_gZbh1wvc;
  SignalVarf sVarf_LIFWLkBW;
  SignalVarf sVarf_1DVejRoD;
  ControlBinop cBinop_dAXTkyMW;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
